@@ -505,7 +505,7 @@ narr_variable <- function(variable) {
   )
   pressure <- c("air", "hgt", "omega", "shum", "tke", "uwnd", "vwnd")
   soil <- c("soill", "soilw", "tsoil")
-  base <- "https://psl.noaa.gov/thredds/fileServer/Datasets/NARR/Dailies/"
+  base <- "https://downloads.psl.noaa.gov//Datasets/NARR/Dailies/"
   if (variable %in% mono) {
     base <- paste0(base, "monolevel/")
     months <- ""
@@ -575,4 +575,4 @@ check_destfile <- function(destfile) {
   } else {
     return(FALSE)
   }
-}
\ No newline at end of file
+}
